﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ryujin
{
    /// <summary>
    /// 弾幕の処理パターンを定義するクラスです。
    /// </summary>
    class ShotPattern
    {
        /// <summary>
        /// プライベートなコンストラクタ。
        /// </summary>
        private ShotPattern()
        {
            ; // 何もしない
        }

        /// <summary>
        /// 弾を発射した敵と自キャラクタとの角度を返します。
        /// </summary>
        /// <param name="shot">弾幕</param>
        /// <returns>角度</returns>
        private static double shotatan2(Shot shot)
        {
            return Math.Atan2(GV.ch.y - shot.parent_enemy.y, GV.ch.x - shot.parent_enemy.x);
        }

        /// <summary>
        /// 1発だけ自キャラクタに向かって直線移動。
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H000(Shot shot)
        {
            if (shot.cnt == 0)
            {
                if (shot.flag != 2)
                {
                    Bullet bullet = shot.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = shot.parent_enemy.blknd2;
                        bullet.angle = shotatan2(shot);
                        bullet.flag = 1;
                        bullet.x = shot.parent_enemy.x;
                        bullet.y = shot.parent_enemy.y;
                        bullet.col = shot.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }

        /// <summary>
        /// 100カウント中に 10発、自キャラクタに向かって直線発射(常に自キャラクタ狙い)
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H001(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt < 100 && shot.cnt % 10 == 0) // 100カウント中 10カウントに 1回
                if (shot.flag != 2) // 敵がいて
                {
                    Bullet bullet = shot.bullets.search_unused_bullet();
                    if (bullet != null) // 弾が登録可能なら
                    {
                        bullet.knd = shot.parent_enemy.blknd2; // 弾登録
                        bullet.angle = shotatan2(shot); // 角度
                        bullet.flag = 1; // フラグ
                        bullet.x = shot.parent_enemy.x; // X座標
                        bullet.y = shot.parent_enemy.y; // Y座標
                        bullet.col = shot.parent_enemy.col; // 色
                        bullet.cnt = 0; // カウンタ
                        bullet.spd = 3.0; // スピード

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
        }

        /// <summary>
        /// 100カウント中に 10発、自キャラクタに向かって直線発射(角度記憶)
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H002(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt < 100 && shot.cnt % 10 == 0)
            {
                if (shot.cnt == 0)
                    shot.base_angle[0] = shotatan2(shot);
                if (shot.flag != 2)
                {
                    Bullet bullet = shot.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = shot.parent_enemy.blknd2;
                        bullet.angle = shot.base_angle[0];
                        bullet.flag = 1;
                        bullet.x = shot.parent_enemy.x;
                        bullet.y = shot.parent_enemy.y;
                        bullet.col = shot.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 3.0;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
            }
        }

        /// <summary>
        /// 100カウント中に 10発、自キャラクタに向かってスピード変化直線発射。
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H003(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt < 100 && shot.cnt % 10 == 0)
                if (shot.flag != 2)
                {
                    Bullet bullet = shot.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = shot.parent_enemy.blknd2;
                        bullet.angle = shotatan2(shot);
                        bullet.flag = 1;
                        bullet.x = shot.parent_enemy.x;
                        bullet.y = shot.parent_enemy.y;
                        bullet.col = shot.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 1.0 + 5.0 / 100.0 * shot.cnt;

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
        }

        /// <summary>
        /// 0.5秒に 1回ずつ円形発射。
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H004(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt < 120 && shot.cnt % 20 == 0)
            {
                double angle = shotatan2(shot);
                for (int i = 0; i < 20; i++)
                {
                    if (shot.flag != 2)
                    {
                        Bullet bullet = shot.bullets.search_unused_bullet();
                        if (bullet != null)
                        {
                            bullet.knd = shot.parent_enemy.blknd2;
                            bullet.angle = angle + MyMath.PI2 / 20.0 * i;
                            bullet.flag = 1;
                            bullet.x = shot.parent_enemy.x;
                            bullet.y = shot.parent_enemy.y;
                            bullet.col = shot.parent_enemy.col;
                            bullet.cnt = 0;
                            bullet.spd = 4.0;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// ばらまきショット。
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H005(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt < 120 && shot.cnt % 2 == 0)
                if (shot.flag != 2)
                {
                    Bullet bullet = shot.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = shot.parent_enemy.blknd2;
                        bullet.angle = shotatan2(shot) + MyMath.rang(Math.PI / 4.0);
                        bullet.flag = 1;
                        bullet.x = shot.parent_enemy.x;
                        bullet.y = shot.parent_enemy.y;
                        bullet.col = shot.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 3.0 + MyMath.rang(1.5);

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }
        }

        /// <summary>
        /// ばらまきショット(減速)。
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H006(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt <= 120 && shot.cnt % 2 == 0)
                if (shot.flag != 2)
                {
                    Bullet bullet = shot.bullets.search_unused_bullet();
                    if (bullet != null)
                    {
                        bullet.knd = shot.parent_enemy.blknd2;
                        bullet.angle = shotatan2(shot) + MyMath.rang(Math.PI / 4.0);
                        bullet.flag = 1;
                        bullet.x = shot.parent_enemy.x;
                        bullet.y = shot.parent_enemy.y;
                        bullet.col = shot.parent_enemy.col;
                        bullet.cnt = 0;
                        bullet.spd = 4.0 + MyMath.rang(2.0);

                        MusicManager musicManager = MusicManager.GetInstance();
                        musicManager.se_flag[0] = 1;
                    }
                }

            for (int i = 0; i < shot.bullets.length(); i++) // 全弾分ループ
            {
                Bullet bullet = shot.bullets[i];
                if (bullet.spd > 1.5) // 速度が 1.5より大きいものがあれば
                    bullet.spd -= 0.04; // 減速
            }
        }

        /// <summary>
        /// みょん弾幕。
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H007(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt <= 150 && shot.cnt % 10 == 0)
            {
                for (int i = 0; i < 20; i++)
                {
                    if (shot.flag != 2)
                    {
                        Bullet bullet = shot.bullets.search_unused_bullet();
                        if (bullet != null)
                        {
                            bullet.knd = 7;
                            bullet.angle = MyMath.PI2 / 20.0 * i;
                            bullet.flag = 1;
                            bullet.x = shot.parent_enemy.x + Math.Cos(Math.PI / 2 + Math.PI / 150 * shot.cnt) * 100.0;
                            bullet.y = shot.parent_enemy.y + Math.Sin(Math.PI / 2 + Math.PI / 150 * shot.cnt) * 100.0;
                            bullet.col = 2;
                            bullet.cnt = 0;
                            bullet.spd = 1.2;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                    }
                }
                for (int i = 0; i < 20; i++)
                {
                    if (shot.flag != 2)
                    {
                        Bullet bullet = shot.bullets.search_unused_bullet();
                        if (bullet != null)
                        {
                            bullet.knd = 7;
                            bullet.angle = MyMath.PI2 / 20.0 * i;
                            bullet.flag = 1;
                            bullet.x = shot.parent_enemy.x + Math.Cos(Math.PI / 2 - Math.PI / 150 * shot.cnt) * 100.0;
                            bullet.y = shot.parent_enemy.y + Math.Sin(Math.PI / 2 - Math.PI / 150 * shot.cnt) * 100.0;
                            bullet.col = 4;
                            bullet.cnt = 0;
                            bullet.spd = 1.2;

                            MusicManager musicManager = MusicManager.GetInstance();
                            musicManager.se_flag[0] = 1;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// ミシャグジさま。
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H008(Shot shot)
        {
            if (shot.cnt >= 0 && shot.cnt < 1200 && shot.cnt % 90 == 0)
            {
                double angle = MyMath.rang(Math.PI);
                for (int j = 0; j < 2; j++) // 途中から 2分裂する弾
                {
                    for (int i = 0; i < 60; i++) // 一度に 60個
                    {
                        if (shot.flag != 2)
                        {
                            Bullet bullet = shot.bullets.search_unused_bullet();
                            if (bullet != null)
                            {
                                bullet.knd = 8; // 8番の弾
                                bullet.angle = angle + MyMath.PI2 / 60 * i; // 円形 60個
                                bullet.flag = 1;
                                bullet.x = shot.parent_enemy.x;
                                bullet.y = shot.parent_enemy.y;
                                bullet.col = shot.parent_enemy.col; // ×4番の色 ○設定した色
                                bullet.cnt = 0;
                                bullet.state = j; // ステータス。0か 1かで回転が変わる
                                bullet.spd = 2.0;

                                MusicManager musicManager = MusicManager.GetInstance();
                                musicManager.se_flag[0] = 1;
                            }
                        }
                    }
                }
            }

            for (int i = 0; i < shot.bullets.length(); i++) // 全弾分
            {
                Bullet bullet = shot.bullets[i];
                if (bullet.flag > 0) // 登録されている弾があれば
                    if (30 < bullet.cnt && bullet.cnt < 120) // 30～120カウントであれば
                    {
                        bullet.spd -= 1.2 / 90; // 90カウントかけて 1.2減らす
                        bullet.angle += (Math.PI / 2) / 90 * ((bullet.state == 0) ? -1 : 1); // 90カウントかけて 90度傾ける
                    }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H009(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H010(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H011(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H012(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H013(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H014(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H015(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H016(Shot shot)
        {
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="shot">弾幕</param>
        private static void shot_bullet_H017(Shot shot)
        {
        }

        /// <summary>
        /// 弾幕を処理します。
        /// </summary>
        /// <param name="shot">弾幕</param>
        public static void do_shot_bullet(Shot shot)
        {
            switch (shot.knd)
            {
                case 0:
                    shot_bullet_H000(shot);
                    break;
                case 1:
                    shot_bullet_H001(shot);
                    break;
                case 2:
                    shot_bullet_H002(shot);
                    break;
                case 3:
                    shot_bullet_H003(shot);
                    break;
                case 4:
                    shot_bullet_H004(shot);
                    break;
                case 5:
                    shot_bullet_H005(shot);
                    break;
                case 6:
                    shot_bullet_H006(shot);
                    break;
                case 7:
                    shot_bullet_H007(shot);
                    break;
                case 8:
                    shot_bullet_H008(shot);
                    break;
                case 9:
                    shot_bullet_H009(shot);
                    break;
                case 10:
                    shot_bullet_H010(shot);
                    break;
                case 11:
                    shot_bullet_H011(shot);
                    break;
                case 12:
                    shot_bullet_H012(shot);
                    break;
                case 13:
                    shot_bullet_H013(shot);
                    break;
                case 14:
                    shot_bullet_H014(shot);
                    break;
                case 15:
                    shot_bullet_H015(shot);
                    break;
                case 16:
                    shot_bullet_H016(shot);
                    break;
                case 17:
                    shot_bullet_H017(shot);
                    break;
                default:
                    Debug.WriteLine("弾幕の種類が異常値(" + shot.knd + ")です");
                    break;
            }
        }
    }
}
